/**
 * This Stabile models a rectangle capped with a
 * semicircle at each end.
 */
public class BarStabile extends Stabile
{
    protected double capRadius;
    protected double capLength;
    protected double sideLength;
    protected double totalPerimeter;
    protected double dom1, dom2, dom3, dom4, dom5;

    /**
     * Create a BarStabile at center point c and
     * with basic size s.  The cap radius is computed
     * to be the basic size divided by 8, and the 
     * side length is computed to be the basic
     * size multipled by PI/4.
     */
    public BarStabile(PPoint c, double s) {
	super(c);
	capRadius = s / 8.0;
	capLength = (capRadius * Math.PI);
	sideLength = s * (7.0 * Math.PI / 8.0);
	totalPerimeter = sideLength * 2.0 + capLength * 2.0;
	dom1 = capLength / 2.0;
	dom2 = dom1 + sideLength;
	dom3 = dom2 + capLength;
	dom4 = dom3 + sideLength;
	dom5 = dom4 + capLength/2.0;
    }

    public double angleToDistance(double phi) {
	throw new RuntimeException("Cannot compute angleToDistance!");
    }

    protected double getCanonicalDistance(double d) {
	// this is kinda cheezy, and should be changed
	while(d < 0.0) {
	    d += totalPerimeter;
	}
	// this part is okay
	if (d > totalPerimeter) {
	    int q = (int)Math.floor(d / totalPerimeter);
	    d = d - (q * totalPerimeter);
	}
	return d;
    }

    protected int getDomain(double cd) {
	if (cd < dom1) {
	    return 1;
	}
	else if (cd < dom2) {
	    return 2;
	}
	else if (cd < dom3) { 
	    return 3;
	}
	else if (cd < dom4) {
	    return 4;
	}
	else return 5;
    }

    public double distanceToAngle(double d) {
	double a = 0.0;
	double cd = getCanonicalDistance(d);
	int dom = getDomain(cd);

	switch(dom) {
	case 1: {
	    a = cd / capRadius;
	    break;
	}

	case 2: {
	    a = Math.PI / 2.0;
	    break;
	}

	case 3: {
	    cd -= (capLength / 2.0);
	    cd -= sideLength;
	    a = cd / capRadius;
	    a += Math.PI / 2.0;
	    break;
	}

	case 4: {
	    a = 3.0 * Math.PI / 2.0;
	    break;
	}

	case 5: {
	    cd -= (capLength / 2.0);
	    cd -= sideLength;
	    cd -= capLength;
	    cd -= sideLength;
	    a = cd / capRadius;
	    a += 3.0 * Math.PI / 2.0;
	    break;
	}
	}
	return a;
    }

    public double distanceToNormal(double dist, boolean inside) {
	double a = distanceToAngle(dist);
	if (inside) a += Math.PI;
	return a;
    }
    
    public PPoint distanceToPoint(double dist) {
	double nx = 0.0;
	double ny = 0.0;
	double a = 0.0;
	double cd = getCanonicalDistance(dist);
	int dom = getDomain(cd);

	switch(dom) {
	case 1: {
	    a = cd / capRadius;
	    nx = Math.cos(a) * capRadius;
	    nx += sideLength / 2.0;
	    ny = Math.sin(a) * capRadius;
	    break;
	}

	case 2: {
	    cd -= (capLength / 2.0);
	    ny = capRadius;
	    nx = (sideLength/2.0) - cd;
	    break;
	}

	case 3: {
	    cd -= (capLength / 2.0);
	    cd -= sideLength;
	    a = cd / capRadius;
	    a += Math.PI / 2.0;
	    nx = Math.cos(a) * capRadius;
	    nx -= sideLength / 2.0;
	    ny = Math.sin(a) * capRadius;
	    break;
	}

	case 4: {
	    cd -= (capLength / 2.0);
	    cd -= sideLength;
	    cd -= capLength;
	    ny = -capRadius;
	    nx = -(sideLength/2.0) + cd;
	    break;
	}

	case 5: {
	    cd -= (capLength / 2.0);
	    cd -= sideLength;
	    cd -= capLength;
	    cd -= sideLength;
	    a = cd / capRadius;
	    a += 3.0 * Math.PI / 2.0;
	    nx = Math.cos(a) * capRadius;
	    nx += sideLength / 2.0;
	    ny = Math.sin(a) * capRadius;
	    break;
	}
	}
	return new PPoint(center, nx, ny);
    }

    public String toString() {
        return "BarStabile[c=" + center + ", caprad=" + capRadius + 
	    ", sideLen=" + sideLength + "]";
    }

}


