import java.awt.*;
import java.awt.event.*;
import java.awt.image.*;
import java.util.*;


public class ColorPicker extends Dialog 
    implements MouseListener, ActionListener, WindowListener
{
    int colorValues[] = { 0xFF, 0xCC, 0x99, 0x66, 0x33, 0 };
    Color selectedColor;
    Component setComponent;
    static Color colors[] = null;

    public Color getSelectedColor() { return selectedColor; }

    class ColorCanvas extends Canvas {
	int cwid, chgt;
	Dimension prefsize; 

	ColorCanvas(int siz) {
	    int len = colorValues.length;
	    prefsize = new Dimension(len * len * siz, len * siz);
	    setSize(prefsize);
	    if (colors == null) {
		colors = new Color[len * len * len];
		int r, g, b, i;
		for(i = 0, r = 0; r < len; r++) {
		    for(g = 0; g < len; g++) {
			for(b = 0; b < len; b++) {
			    int col = (colorValues[r] << 16) |
				      (colorValues[g] << 8) | colorValues[b];
			    colors[i] = new Color(col);
			    i += 1;
			}
		    }
		}
	    }
	}

	public Dimension getPreferredSize() {
	    return prefsize;
	}

	public void paint(Graphics g) {
	    int r, c;
	    int len = colorValues.length;
	    int wid = getSize().width;
	    int hgt = getSize().height;
	    cwid = wid / (len * len);
	    chgt = hgt / len;
	    g.setColor(Color.lightGray);
	    g.fillRect(0, 0, wid, hgt);
	    for(r = 0; r < len; r++) {
		for(c = 0; c < (len * len); c++) {
		    g.setColor(colors[r * len * len + c]);
		    g.fillRect(c * cwid + 2, r * chgt + 2, cwid - 2, chgt - 2);
		}
	    }
	    g.setColor(Color.black);
	    g.drawRect(0,0, wid-1, hgt - 1);
	}
    }

    ColorCanvas myCanvas;
    TextField rtext;
    TextField gtext;
    TextField btext;
    Label  sample;
    Label  samphex;

    static final Frame getFrameParent(Component c) {
	if (c == null) return null;
	else if (c instanceof Frame) return (Frame)c;
	else return getFrameParent(c.getParent());
    }

    public ColorPicker(Component par, Component setC) {
	super(getFrameParent(par));
	setComponent = setC;
	Panel innerPanel;
	setBackground(Color.lightGray);
	setLayout(new BorderLayout(3,3));
	innerPanel = new Panel();
	innerPanel.setLayout(new BorderLayout(2,2));

	Font f10 = new Font("SansSerif", Font.PLAIN, 10);
	innerPanel.setFont(f10);

	myCanvas = new ColorCanvas(13);
	innerPanel.add("Center", myCanvas);
	Panel cPanel = new Panel();
	Label caption;
	caption = new Label("Please select a Color");
	innerPanel.add("North", caption);
	caption = new Label("Red:", Label.RIGHT);
	rtext = new TextField(3);
	cPanel.add(caption);
	cPanel.add(rtext);
	caption = new Label(" Green:", Label.RIGHT);
	gtext = new TextField(3);
	cPanel.add(caption);
	cPanel.add(gtext);
	caption = new Label(" Blue:", Label.RIGHT);
	btext = new TextField(3);
	cPanel.add(caption);
	cPanel.add(btext);
	caption = new Label(" Sample:", Label.RIGHT);
	sample = new Label("     ");
	cPanel.add(caption);
	cPanel.add(sample);
	sample.setBackground(Color.black);
	samphex = new Label("(000000)    ");
	cPanel.add(samphex);
	
	innerPanel.add("South", cPanel);
	add("Center", innerPanel);

	Panel aPanel = new Panel();
	aPanel.setLayout(new GridLayout(1,5));
	caption = new Label("   ");
	Button ok = new Button("Okay");
	aPanel.add(caption);
	aPanel.add(ok);
	ok.setActionCommand("ok");
	caption = new Label("   ");
	Button cancel = new Button("Cancel");
	cancel.setActionCommand("cancel");
	aPanel.add(caption);
	aPanel.add(cancel);
	caption = new Label("   ");
	aPanel.add(caption);
	add("South", aPanel);

	ok.addActionListener(this);
	cancel.addActionListener(this);
	myCanvas.addMouseListener(this);
	rtext.addActionListener(this);
	gtext.addActionListener(this);
	btext.addActionListener(this);
	addWindowListener(this);
	pack();
	setLocation(120,120);
	// setResizable(false);
    }
	
    public String getHexString(Color c) {
	int v;
	StringBuffer hb = new StringBuffer();
	hb.append("(");
	v = c.getRed();
	if (v < 16) hb.append('0');
	hb.append(Integer.toString(v,16));
	v = c.getGreen();
	if (v < 16) hb.append('0');
	hb.append(Integer.toString(v,16));
	v = c.getBlue();
	if (v < 16) hb.append('0');
	hb.append(Integer.toString(v,16));
	hb.append(")    ");
	return hb.toString();
    }

    public void show() {
	if (setComponent != null) {
	    Color c = setComponent.getBackground();
	    sample.setBackground(c);
	    rtext.setText("" + c.getRed());
	    gtext.setText("" + c.getGreen());
	    btext.setText("" + c.getBlue());
	    samphex.setText(getHexString(c));
	}
	super.show();
    }

    public int getLimitedInt(String s) {
	int i = 0;
	try {
	    i = Integer.parseInt(s);
	    if (i < 0) i = 0;
	    if (i > 255) i = 255;
	}
	catch (NumberFormatException nfe) { }
	return i;
    }
	    
    public void grabValuesAndSet() {
	String rs = rtext.getText().trim();
	String gs = gtext.getText().trim();
	String bs = btext.getText().trim();

	int rv = getLimitedInt(rs);
	int gv = getLimitedInt(gs);
	int bv = getLimitedInt(bs);
       
	rtext.setText(rv + "");
	gtext.setText(gv + "");
	btext.setText(bv + "");

	int col = (rv << 16) | (gv << 8) | bv;
	Color c2 = new Color(col);
	sample.setBackground(c2);
	samphex.setText(getHexString(c2));	
    }

    public void actionPerformed(ActionEvent e) {
	String cmd = e.getActionCommand();
	Component src = (Component)(e.getSource());
	if (src instanceof TextField) {
	    grabValuesAndSet();
	}
	else if (cmd.equals("ok")) {
	    grabValuesAndSet();
	    selectedColor = sample.getBackground();
	    if (setComponent != null)
		setComponent.setBackground(selectedColor);
	    setVisible(false);
	}
	else if (cmd.equals("cancel")) {
	    selectedColor = null;
	    setVisible(false);
	}
    }

    public void windowOpened(WindowEvent e) {}
    public void windowClosing(WindowEvent e) {
	selectedColor = null;
	setVisible(false);
    }
    public void windowClosed(WindowEvent e) {}
    public void windowIconified(WindowEvent e) {}
    public void windowDeiconified(WindowEvent e) {}
    public void windowActivated(WindowEvent e) {}
    public void windowDeactivated(WindowEvent e) {}

    public void mouseClicked(MouseEvent e) { }
    public void mousePressed(MouseEvent e) { }
    public void mouseReleased(MouseEvent e) {
	int len = colorValues.length;
	int x = e.getX();
	int y = e.getY();
	if (e.getComponent() != myCanvas) return;
	int r = y / myCanvas.chgt;
	int c = x / myCanvas.cwid;
	if (r < 0 || r >= len) return;
	if (c < 0 || c >= (len*len)) return;

	int rv = colorValues[r];
	int gv = colorValues[c/len];
	int bv = colorValues[c%len];

	rtext.setText("" + rv);
	gtext.setText("" + gv);
	btext.setText("" + bv);
	sample.setBackground(colors[(r * len * len) + c]);
	samphex.setText(getHexString(colors[(r * len * len) + c]));
    }
    public void mouseEntered(MouseEvent e) {}
    public void mouseExited(MouseEvent e) {}


    /*** Main for testing...
    static Frame f;
    static Button btn;
    static ColorPicker cp;
    public static void main(String [] args) {
	f = new Frame("ColorPicker test");
	btn = new Button("Click me");
	f.add("Center", btn);
	cp = new ColorPicker(btn,btn);
	btn.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent e) {
		    cp.setModal(true);
		    cp.show();
		    Color sc = cp.getSelectedColor();
		    if (sc != null) btn.setBackground(sc);
		} });
	f.pack();
	f.show();
    }
     */

}


