import java.awt.*;
import java.awt.event.*;
import java.awt.image.*;
import java.util.*;
import java.util.zip.*;
import java.io.*;
import util.*;

public class DesignCanvas extends Canvas
{
    public static final int FORMAT_PPM = 1;
    public static final int FORMAT_TGA = 2;

    int originalSize;
    Vector images;
    public Vector designs;
    Color xorColor;
    Graphics xorGraphics;

    DesignCanvas() {
	images = new Vector();
	designs = new Vector();
	originalSize = 0;
	xorColor = new Color(0x066, 0x066, 0x066);
	xorGraphics = null;
    }

    public DesignCanvas(int siz) {
	this();
	originalSize = siz;
	setSize(siz,siz);
    }

    public void addImage(Image i, Design d) {
	images.addElement(i);
	designs.addElement(d);
	repaint();
    }

    public int countImages() { return images.size(); }

    public void removeImage(int num) {
	if (num >= 0 && num < images.size()) {
	    images.removeElementAt(num);
	    designs.removeElementAt(num);
	    repaint();
	}
	return;
    }

    public void removeAllImages() {
	images.removeAllElements();
	designs.removeAllElements();
	repaint();
    }

    public void paint(Graphics g) {
	Enumeration e;
	if (xorGraphics == null) {
	    xorGraphics = getGraphics();
	}
	for(e = images.elements(); e.hasMoreElements(); ) {
	    Image i = (Image)(e.nextElement());
	    g.drawImage(i, 0, 0, this);
	}
    }
    
    public boolean writeData(Design [] ds, OutputStream os, Color bg, int fmt) {
	int [] pixels = null;
	int i;
	Design d;
	Color bgsave;
	ImageBlotter ib;
	ds[0].message("Re-rendering high-quality image.");
	for(i = 0; i < ds.length; i++) {
	    d = ds[i];
	    d.run();
	    ib = d.getBlotter();
	    bgsave = ib.getBackground();
	    ib.setColors(null, bg);
	    d.message("Adding to composite image..");
	    pixels = ib.addToMemoryImage(pixels);
	    ib.release();
	    d.message("Done with design " + (i+1) + ".");
	    ib.setColors(null, bgsave);
	}
	return writeData(os, bg, pixels, fmt);
    }

    public boolean writeData(OutputStream os, Color bg, int [] pixels, int fmt) {
	BufferedOutputStream bos = new BufferedOutputStream(os);
	boolean ret = false;
	int siz = originalSize;
	if (pixels == null) {
	    Image img = createImage(siz, siz);
	    Graphics g = img.getGraphics();
	    g.setColor(bg);
	    g.fillRect(0, 0, siz, siz);
	    paint(g);
	    pixels = new int[siz * siz];
	    PixelGrabber grabber = new PixelGrabber(img,0,0,siz,siz,pixels,0,siz);
	    try {
		grabber.grabPixels();
		ret = true;
	    } catch (InterruptedException ie) { }
	}
	else {
	    ret = true;
	}
	if (ret) {
	    int i;
	    switch(fmt) {
	    case FORMAT_PPM: {
		StringBuffer sb = new StringBuffer();
		sb.append("P6\n");
		sb.append(siz);
		sb.append(" ");
		sb.append(siz);
		sb.append("\n255\n");
		try {
		    bos.write(sb.toString().getBytes());
		    for(i = 0; i < pixels.length; i++) {
			bos.write((pixels[i] >> 16)&0xff);
			bos.write((pixels[i] >> 8)&0xff);
			bos.write((pixels[i])&0xff);
		    }
		    // bos.close();
		    bos.flush();
		}
		catch (IOException ie2) {
		    ret = false;
		}
	    }
	    break;
	    
	    case FORMAT_TGA: {
		byte header[] = new byte[18];
		for(i = 0; i < header.length; i++) header[i] = (byte)0;
		header[2] = (byte)2;
		header[12] = (byte)(siz%256);
		header[13] = (byte)(siz/256);
		header[14] = (byte)(siz%256);
		header[15] = (byte)(siz/256);
		header[16] = (byte)24;
		try {
		    int r;
		    bos.write(header);
		    for(r = (siz-1); r >= 0; r--) {
			for(i = 0; i < siz; i++) {
			    bos.write((pixels[i+(r*siz)])&0xff);
			    bos.write((pixels[i+(r*siz)] >> 8)&0xff);
			    bos.write((pixels[i+(r*siz)] >> 16)&0xff);
			}
		    }
		    // bos.close();
		    bos.flush();
		}
		catch (IOException ie2) {
		    ret = false;
		}
	    }
	    break;

	    default: {
		throw new RuntimeException("Unsupported image format.");
	    }
	    }
	}
	return ret;
    }

    Point rbStart;
    Point rbCurrent;
    PShape rbShape;

    /* put rubber-banding support here */

    /**
     * Main for testing..
     * args[0] = stabile radius
     * args[1] = mobile radius
     * args[2] = mobile pen radius
     * args[3] = inside(t) or outside (f or absent)

    static Frame f;
    static DesignCanvas dc;
    public static void main(String [] args) {
	if (args.length < 3) {
	    System.out.println("Usage: java DesignCanvas sradius mradius pradius true/false revs");
	    System.exit(0);
	}

	double sradius = 0.0;
	double mradius = 0.0;
	double pradius = 0.0;
	int revs = 30;
	boolean inside = false;
	double canvasSizeWorld;
       
	try {
	    sradius = Double.valueOf(args[0]).doubleValue();
	    mradius = Double.valueOf(args[1]).doubleValue();
	    pradius = Double.valueOf(args[2]).doubleValue();
	    if (args.length > 3) 
		inside = Boolean.valueOf(args[3]).booleanValue();
	    if (args.length > 4) 
		revs = Integer.parseInt(args[4]);
	}
	catch (NumberFormatException nfe) {
	    System.out.println("Bad number: " + nfe);
	    System.exit(0);
	}

	int siz = 960;
	f = new Frame("DesignCanvas Test");
	dc = new DesignCanvas(siz);
	dc.setBackground(Color.white);
	f.add("Center", dc);


	Stabile s;
	Mobile m;
	if (inside) 
	    canvasSizeWorld = 2.25 * sradius;
	else 
	    canvasSizeWorld = 2.25 * (sradius + (2 * mradius));

	System.out.println("sradius=" + sradius);
	System.out.println("mradius=" + mradius);
	System.out.println("pradius=" + pradius);
	System.out.println("canvasSizeWorld=" + canvasSizeWorld);
	System.out.println("revs=" + revs);

	s = new CircleStabile(new PPoint(), sradius);
	m = new CircleMobile(mradius, pradius);
	double ia = 0.0;
	Design des = new Design(m, s, ia, revs, canvasSizeWorld,
				siz, 1.05, 2.05, 500, inside);
	
	des.addObserver(new Observer() {
		public void update(Observable o, Object arg) {
		    if (arg instanceof Number)
			System.out.print(arg.toString() + ",");
		    else if (arg instanceof String) {
			System.out.println(arg.toString());
			Image i = ((Design)o).getImage(dc, Color.green.darker(),
					       Color.white, false);
			dc.addImage(i);
			try {
			    FileOutputStream fos = new FileOutputStream("try.ppm.gz");
			    if (dc.writeData(new GZIPOutputStream(fos), 
					    Color.white, DesignCanvas.FORMAT_PPM)) {
				System.out.println("Did write!");
			    }
			    else {
				System.out.println("Write failed!");
			    }
			}
			catch (IOException ie) {
			    System.out.println("File open failed: " + ie);
			}
		    }
		} });
	
	System.out.println("About to start thread..");
	Thread t1 = new Thread(des);
	f.pack();
	t1.start();
	f.show();
    }
     */
}

