import java.awt.*;
import java.io.*;
import java.awt.image.*;

/**
 * SaveManager for saving to a local file.  This class is 
 * not appropriate for applets.
 */
public class FileSaveManager extends SaveManager implements FilenameFilter {
    public String formatMessage(Object arg) {
	return arg.toString();
    }

    public void performSave(DesignCanvas dc, Color bg) {
	Component p;
	getDesigns(dc);
	if (designs == null || designs.length == 0) {
	    setChanged();
	    notifyObservers("No designs drawn; not saving.");
	    return;
	}
	for(p = dc.getParent(); p != null && !(p instanceof Frame); 
	    p = p.getParent());
	if (p != null && (p instanceof Frame)) {
	    FileDialog fd = new FileDialog((Frame)p, "Save Targa (.TGA) Image File", 
					   FileDialog.SAVE);
	    fd.setFilenameFilter(this); 
	    fd.show();
	    String fn = fd.getFile();
	    String fdir = fd.getDirectory();
	    if (fn == null || fdir == null) return;
	    File f2 = new File(fdir,fn);
	    if (fn != null) {
		try {
		    FileOutputStream fos = new FileOutputStream(f2);
		    if (dc.writeData(designs, fos, bg, DesignCanvas.FORMAT_TGA)) {
			setChanged();
			notifyObservers("Write to " + fn + " succeeded.");
		    }
		    else {
			setChanged();
			notifyObservers("Write to " + fn + " failed.");
		    }
		    fos.close();
		}
		catch (IOException ie) {
			setChanged();
			notifyObservers("Write to " + fn + " failed: " + ie);
		}
	    }
	}
	return;
    }

    public boolean accept(File dir, String name) {
	return name.toLowerCase().endsWith(".tga");
    }
}





