/**
 * This abstract class is the basis for the moving
 * part of a spirograph rendering.  It has to be able
 * to do several things: 1. given an angle, compute the
 * total distance along the perimeter (roll distance),
 * 2. given a normal angle and a perimeter point and 
 * a distance, compute the location of the pen point.
 * <p>
 * Note: in this class, as in Stabile, everything is
 * computed with world coordinates.
 */

public abstract class Mobile implements PShape {
    double radius;
    double penRadius;

    public Mobile(double s, double pr) {
	radius = s;
	penRadius = pr;
    }

    /**
     * Given a point on the perimeter, the angle normal to the shape
     * on which the mobile is rolling, and a roll distance along
     * the perimeter, return the point at which the pen would be
     * sitting.
     */
    public abstract void 
	findPen(PPoint pp, double norm, double dist, boolean inside,
		PPoint result);
}


