import java.applet.*;
import java.awt.*;
import java.awt.event.*;
import java.awt.image.*;
import java.util.*;
import java.io.*;
import java.net.*;

/**
 * This applet expects two applet parameters:
 *
 *   1. help or helpURL
 *      Full absolute URL of the HTML page that serves
 *      as the help for the applet.
 *
 *   2. CGIURL or baseURL
 *      The base URL for the reflector CGI (see CGISaveManager
 *      for details.)
 */

public class SpiroApplet extends Applet 
    implements ActionListener, Observer
{
    ControlPanel cp;
    DesignPanel dp;
    Frame mainFrame;
    int newWinX, newWinY;
    URL baseURL = null;
    SaveManager savemgr = null;
    boolean listEm;

    Font btnFont;
    Font lblFont;
    
    int sizes[] = { 512, 640, 800, 1024 };
    String sizeNames[] = { "Small", "Medium", "Large", "Huge" };

    public SaveManager getSaveMgr() {
	if (savemgr == null) {
	    // System.out.println("In getSaveMGR..");
	    savemgr = new CGISaveManager();
	    if (((CGISaveManager)savemgr).setURL(getBaseURL(),null)) {
		savemgr.addObserver(this);
	    }
	    else {
		savemgr = null;
	    }
	}
	return savemgr;
    }

    public URL getBaseURL() {
	if (baseURL == null) {
	    String bu = getParameter("baseURL");
	    if (bu == null)
		bu = getParameter("CGIURL");
	    if (bu == null)
		bu = getParameter("SpiroURL");
	    if (bu != null) try {
		// System.out.println("Creating base URL: " + bu);
		baseURL = new URL(bu);
	    }
	    catch (Exception e) { 
		System.out.println("Base URL failed: " + e);
	    }
	}
	return baseURL;
    }

    public void init() {
	btnFont = new Font("SansSerif", Font.BOLD, 14);
	lblFont = new Font("SansSerif", Font.ITALIC, 10);
	newWinX = 14;
	newWinY = 14;
	setLayout(new GridLayout(0,1,2,2));
	setBackground(new Color(0x0FFCC99));
	Panel p1;
	Button btn;
	Label lbl;
	for(int i = 0; i < sizes.length; i++) {
	    p1 = new Panel();
	    p1.setLayout(new FlowLayout(FlowLayout.LEFT));
	    btn = new Button(sizeNames[i]);
	    btn.setFont(btnFont);
	    btn.setActionCommand(sizeNames[i]);
	    btn.addActionListener(this);
	    p1.add(btn);
	    lbl = new Label(" (" + sizes[i] + "x" + sizes[i] + ")", Label.RIGHT);
	    lbl.setFont(lblFont);
	    p1.add(lbl);
	    add(p1);
	}
	try {
	    String list_em = getParameter("list");
	    if (list_em == null)
	    list_em = getParameter("preset");
	    if (list_em == null)
		list_em = getParameter("presetlist");
	    if (list_em == null || list_em.equals("0") || list_em.equals("false"))
		listEm = false;
	    else
		listEm = true;
	}
	catch (NullPointerException npe) { }
    }


    public void actionPerformed(ActionEvent e) {
	String nam = e.getActionCommand();
	int i;
	for(i = 0; i < sizes.length; i++) {
	    if (sizeNames[i].equals(nam)) break;
	}
	int siz = sizes[i];
	
	String title = "Spirograph " + nam + " Design  (";
	title += siz;
	title += "x" + siz;
	title += ")";
	Frame f = new Frame(title);
	DesignPanel dp = new DesignPanel(siz, getSaveMgr());
	f.add("Center", dp);
	ControlPanel cp = new ControlPanel(dp, (double)siz, f, this, listEm);
	f.add("West", cp);
	f.pack();
	f.setResizable(false);
	f.setLocation(newWinX, newWinY);
	newWinX += 12; newWinY += 10;
	canvasesUp += 1;
	f.show();
	cp.applyPreset(0);
    }

    public void update(Observable o, Object arg) {
	if (arg != null && arg instanceof URL) {
	    getAppletContext().showDocument((URL)arg, "sd");
	    // tf.setText(arg.toString());
	    // System.out.println("Interrupting TFW");
	    // tfWatcher.interrupt();
	}
	return;
    }

    public int canvasesUp = 0;

    /******************************************************************/
    /* stuff below this line is solely related to running SpiroApplet */
    /* as an application.                                             */

    static Frame f;
    static SpiroApplet sa;
    public static void main(String []args) {
	f = new Frame("Spiro 0.98");
	sa = new SpiroApplet();
	sa.savemgr = new FileSaveManager();
	if (args.length > 0) sa.listEm = true;
	f.add("Center",sa);
	f.addWindowListener(new WindowAdapter() {
		public void windowClosing(WindowEvent e) {
		    if (sa.canvasesUp > 0) {
			try {
			    Toolkit.getDefaultToolkit().beep();
			} catch (SecurityException se) { }
		    }
		    else 
			System.exit(0);
		}
	    });
	sa.init();
	sa.start();
	f.pack();
	f.show();
    }
}
