#!/usr/bin/perl
#
# spiro - CGI script for all phases of the spiro
# application
#
# This CGI has several separate tasks:
# 
# 1. Deliver files - 
#    If the PATH_INFO variable is set, then we
#    simply return that file, with the appropriate
#    MIME type.
#
# 2. Accept uploads -
#    If the QUERY_STRING is of the form  "put=NNNN" then we
#    accept the gzipped PPM data and store it to a file named
#    temp/NNNN.ppm.gz.
#
# 3. Produce GIF downloads -
#    If the QUERY_STRING is of the form "get=NNNN" then we
#    pass the file temp/NNNN.ppm.gz through gunzip, ppmquant,
#    and ppmtogif to produce output of MIME type image/gif.
#
# 4. Produce GIF download page -
#    If the QUERY_STRING is of the form "page=NNNN" then we
#    create a little HTML page with the <img> reference to
#    the image (see #3, above) and instructions on how to
#    save the image.
#
# 5. Do nothing.
#

# Subr to send HTTP information: takes one argument, MIME type.
sub SendHeaders {
    my $mimetype = shift;
    print "Content-type: " . $mimetype . "\n\n";
    return;
}


# Step 0 - grab some params
$PathInfo = $ENV{'PATH_INFO'};
$QueryString = $ENV{'QUERY_STRING'};
$ScriptURL = $ENV{'SCRIPT_URL'};
$CodeNumber = 0;
$mode = 5;
$FileName = "";

if (length($PathInfo) > 0) {
    $mode = 1;
    $PathInfo =~ tr/;&|\$ \t()/pppppp/;
    $FileName = "." . $PathInfo;
}
elsif (length($QueryString) > 0) {
    if ($QueryString =~ m/get[=-]([0-9a-f]+)/) {
	$CodeNumber = $1;
	$mode = 3;
    }
    elsif ($QueryString =~ m/put[=-]([0-9a-f]+)/) {
	$CodeNumber = $1;
	$mode = 2;
    }
    elsif ($QueryString =~ m/page[=-]([0-9a-f]+)/) {
	$CodeNumber = $1;
	$mode = 4;
    }
    else {
	$mode = 5;
    }
}



if ($mode == 1) {
    $mt = "application/octet-stream";
  SWITCH: {
      if ($FileName =~ m/[.]jar$/i) { $mt = "application/java-archive"; last SWITCH; }
      if ($FileName =~ m/[.]html$/i) { $mt = "text/html"; last SWITCH; }
      if ($FileName =~ m/[.]htm$/i) { $mt = "text/html"; last SWITCH; }
      if ($FileName =~ m/[.]txt$/i) { $mt = "text/plain"; last SWITCH; }
      if ($FileName =~ m/[.]gif$/i) { $mt = "image/gif"; last SWITCH; }
      if ($FileName =~ m/[.]jpg$/i) { $mt = "image/jpeg"; last SWITCH; }
      if ($FileName =~ m/[.]cla$/i) { $mt = "application/java-class"; last SWITCH; }
  }
    SendHeaders($mt);
    
    open(PFILE,$FileName) or die("Could not open " . $FileName);
    $buf = "";
    while (read(PFILE,$buf,1024)) {
	print $buf;
    }
    close(PFILE);
}
elsif ($mode == 2) {
    $FileName = "./temp/" . $CodeNumber . ".ppm.gz";
    open(OFILE, ">" . $FileName) or die("Could not write to " . $FileName);
    $buf = "";
    while(read(STDIN,$buf,1024)) {
	print OFILE $buf;
    }
    close(OFILE);

    SendHeaders("text/plain");
    print "Okay, accepted " . $FileName . "\n\n";
    system("find ./temp -mtime +2 -print | xargs rm");
}
elsif ($mode == 3) {
    sleep(1);
    $FileName = "./temp/" . $CodeNumber . ".ppm.gz";
    open(GFILE, "gunzip -c $FileName | ./ppmquant -fs 224 | ./ppmtogif |") or die("Could not create translation pipe for " . $FileName);
    SendHeaders("image/gif");
    $buf = "";
    while(read(GFILE,$buf,1024)) {
	print $buf;
    }
    close(GFILE);
}
elsif ($mode == 4) {
    SendHeaders("text/html");
    $ImgURL = $ScriptURL . "?get=" . $CodeNumber;
print <<foobar19
<html>
<head>
<title>Spiro Applet Image</title>
</head>
<body bgcolor="FFFFCC">
<h2>Spiro Applet Generated Image</h2>
<center>
<img border=2 src="$ImgURL" hspace=5 vspace=5>
</center>
<ul>
<h3>Saving the Image</h3>
<ul>
To save this image to a file, use the facilities of your web browser.
<li>Netscape Navigator or Communicator<br>
Right-click on the image, and select <tt>Save Image as..</tt> from the 
popup menu.  A file browser box will come up.  Type in a name ending
with ".gif" and click on Save.
<p>
<li>Microsoft Internet Explorer<br>
Right-click on the image, and select <tt>Save Picture As..</tt> from
the popup menu.  A file browser box will come up.  Type in  a name
eding with ".gif" and click on Save.
</ul>
<p>
<h3>Printing the Image</h3>
<ul>
To print the image, simply print this page, or follow the directions below.
<li>Netscape Navigator or Communicator<br>
Right-click on the image, and select <tt>View Image (...)</tt> from the 
popup menu.  The image will appear by itself in this window.  Then select
File->Print from the main menu bar.
<p>
<li>Microsoft Internet Explorer<br>
Click in the location bar at the top of the Explorer window.
Near the end of the location should appear the word "page" followed
by an equal sign and some numbers.  Change the word "page" to the 
word "get", and then hit the Enter key.
The image will appear by itself in this window.  Then select
File->Print from the main menu bar.
</ul>
</ul>
<hr>
<center>
<small>Spiro Applet and CGI Reflector by <a href="mailto:ziring\@home.com">Neal Ziring</a>, last major changes May 2000.<br>
Copyright 2000 - Neal Ziring
</small></center>
</body>
</html>
foobar19


}

exit 0;
