package util;

import java.awt.*;

public class ImageBlot {
    /**
     * Note: while these are doubles, they are values in image
     * coordinate space, NOT world coordinate space.
     */
    public double fgrad;
    public double bgrad;
    // double eccentricity;
    
    public ImageBlot(double solidRadius, double fadeRadius)
    {
	fgrad = solidRadius;
	bgrad = fadeRadius;
	if (bgrad == 0.0) bgrad = fgrad;
    }

    public ImageBlot(int sR, int fR) {
	this((float)sR, (float)fR);
    }

    /**
     * Apply the spot to the given pixel, relative to the
     * given spot center coordinates.  Return the new pixel
     * value (which might be the same as the old pixel value).
     * Foreground = 1.0, Background=0.0.
     */
    public float applyToPixel(float oldpixel,
			    double cx, double cy,
			    double px, double py)
    {
	double dist = Math.sqrt((cx - px)*(cx - px) +
				(cy - py)*(cy - py));
	if (dist <= fgrad) return 1.0f;
	else if (dist >= bgrad) return oldpixel;
	else {
	    float pv = 1.0f - (float)((dist - fgrad) / (bgrad - fgrad));
	    return ((oldpixel > pv)?(oldpixel):(pv));
	}
    }
}

		     

